@@ -945,6 +945,8 @@ gdip_metafile_clone (GpMetafile *metafile, GpMetafile **clonedmetafile)
 	}
 
 	mf->base = *base;
+	GdipFree(base);
+	base = NULL;
 
 	memcpy (&mf->metafile_header, &metafile->metafile_header, sizeof (MetafileHeader));
 	if (metafile->length > 0) {
@@ -1719,7 +1721,8 @@ GdipCreateMetafileFromWmf (HMETAFILE hWmf, BOOL deleteWmf, GDIPCONST WmfPlaceabl
 		if (wmfPlaceableFileHeader) {
 			status = GdipGetMetafileHeaderFromWmf (hWmf, wmfPlaceableFileHeader, &(*metafile)->metafile_header);
 			if (status != Ok) {
-				GdipFree (*metafile);
+				gdip_metafile_dispose (*metafile);
+				*metafile = NULL;
 				return status;
 			}
 		}
@@ -1730,7 +1733,7 @@ GdipCreateMetafileFromWmf (HMETAFILE hWmf, BOOL deleteWmf, GDIPCONST WmfPlaceabl
 	case MetafileTypeEmfPlusOnly:
 	case MetafileTypeEmfPlusDual:
 	default:
-		GdipFree (*metafile);
+		gdip_metafile_dispose (*metafile);
 		*metafile = NULL;
 		return GenericError;
 	}
