@@ -865,7 +865,9 @@ gdip_combine_exclude (GpRegion *region, GpRectF *rtrg, int cntt)
 			if (alltrgrects) {
 				GdipFree (alltrgrects);
 			}
-
+			if (allsrcrects) {
+				GdipFree(allsrcrects);
+			}
 			return status;
 		}
 	}
@@ -2105,7 +2107,7 @@ GpStatus WINGDIPAPI
 GdipGetRegionScansI (GpRegion *region, GpRect *rects, INT *count, GpMatrix *matrix)
 {
 	GpStatus status;
-	GpRectF *rectsF;
+	GpRectF *rectsF = NULL;
 	UINT scansCount;
 
 	if (!region || !count || !matrix)
@@ -2124,14 +2126,20 @@ GdipGetRegionScansI (GpRegion *region, GpRect *rects, INT *count, GpMatrix *matr
 	}
 
 	status = GdipGetRegionScans (region, rectsF, count, matrix);
-	if (status != Ok)
+	if (status != Ok) {
+		if (rectsF)
+			free(rectsF);
 		return status;
+	}
 		
 	if (rects) {
 		for (int i = 0; i < scansCount; i++)
 			gdip_Rect_from_RectF (&rectsF[i], &rects[i]);
 	}
 
+	if (rectsF)
+		free(rectsF);
+	
 	return Ok;
 }
 
